﻿IF EXISTS (SELECT * FROM sysobjects WHERE type = 'P' AND name = 'ReportTimeSellAnalysisByEmployees')
	BEGIN
		PRINT 'Dropping Procedure ReportTimeSellAnalysisByEmployees'
		DROP  Procedure  ReportTimeSellAnalysisByEmployees
	END
GO

PRINT 'Creating Procedure ReportTimeSellAnalysisByEmployees'
GO
/******************************************************************************
**		File: ReportTimeSellAnalysisByEmployees.sql
**		Name: ReportTimeSellAnalysisByEmployees
**
**         
**		Return values:
**
**		Called by:   
**              
**		Parameters:
**		Input							Output
**      ----------						-----------
**		Auth: Sergey Pelevin			Date: 25.07.14
*******************************************************************************
**		Change History
*******************************************************************************
**		Date:		Author:		Description:
**		--------	--------	-------------------------------------------
**		01.08.14	GNV			Отображение "Фамилия И.О." вместо поля "Хранить как"
**		04.08.14	GNV			Добавление нужной сортировки в графики
**		12.09.14	SAP			Доработка фильтров по квалификации и подразделению
*******************************************************************************/
CREATE PROCEDURE [dbo].[ReportTimeSellAnalysisByEmployees]
	@OwnCompanyID int, --ID собственной компании (NULL-все)
	@DivisionSelector int, --Подразделение (0 - не важно, 2 - из списка)
	@DivisionID int, --ID подразделения (NULL - не указано)
	@QualificationSelector int, --Квалификация (0 - не важно, 2 - из списка)
	@QualificationID int, --ID квалификации (NULL - не указана)
	@EmployeeState int, --Статус сотрудников (0 - работающие, 1 - все, вкл. уволенных)
	@EmployeeSelector int, --Сотрудники (0 - все, 1 - Вкл. в список, 2 - Кроме вкл. в список)
	@EmployeeXml	xml, --Список сотрудников
	@CurrencySelector int, --Валюта отчета (0 - национальная, 1 - учетная)
	@PeriodSelector int, --Период (0 - текущий месяц, 1 - прошлый месяц, 2 - текущий квартал, 3 - прошлый квартал
						-- 4 - текущий год, 5 - прошлый год, 6 - за период
	@PeriodStart date, -- Начало периода
	@PeriodEnd date, --Конец периода
	@IncludingSentToApprove bit, -- Включать отправленное на утверждение (0 - не включать, 1 - включать)
	@ScaleSelector int,	-- Отображать в (0 - единицах, 1 - тысячах, 2 - миллионах, 3 - автовыбор)
	@StateOfFinanceDocuments int, -- Состояние фин. документов (0 - калькуляции учтены, 
									--1 - только утвержденные, 2 - включая черновики)
	@GroupBy int --Группировать по (0 - без группировки, 1 - по подразделению, 2 - по квалификации)
AS

	SET NOCOUNT ON

	DECLARE 
		@PeriodName TName

	
	--*******************************************************
	--проверка на права	
	IF  dbo.adfHasAccessRightByName('ViewTimeSellAnalysisByEmployeesReport') = 0 
	BEGIN
		RAISERROR('#NotEnoughPermissions', 18, -1)
		RETURN 1
	END

	--*******************************************************
	--Подготовка данных из фильтров

	--Установка периода		
	EXEC ReportGetPeriod 
		@PeriodSelector,		-- Тип выбранного периода
		0,
		@PeriodStart OUTPUT,	-- Начало периода
		@PeriodEnd	 OUTPUT,	-- Конец периода
		@PeriodName  OUTPUT		-- Название периода

	--IF @DivisionSelector = 0
	--	SET @DivisionID = NULL

	--IF @QualificationSelector = 0
	--	SET @QualificationID = NULL


	DECLARE	
		@Value TVariant, 
		@StandardEfficiency decimal(18,2)
	EXEC adfParameterGet 'StandardEfficiency', 'Pjm7', @Value OUTPUT
	SET @StandardEfficiency = CAST(@Value as decimal(18,2)) * 100	


	--*******************************************************
	--Временная таблица	
	DECLARE @MainTable TABLE
	(
		[Order] int,
		[EmployeeID] int,		
		[DivisionID] int,
		[QualificationID] int,		
		[UtilizationPlanned] decimal(18,2),
		[UtilizationReal] decimal(18,2),
		[UtilizationDeviation] decimal(18,2),
		[UtilizationDeviationPercent] decimal(18,2),
		[UtilizationAverageMonth] decimal(18,2),
		[UtilizationAverageDay] decimal(18,2),
		[RealizationPlanned] decimal(18,2),
		[RealizationReal] decimal(18,2),
		[RealizationDeviation] decimal(18,2),
		[RealizationDeviationPercent] decimal(18,2),
		[RealizationAverageMonth] decimal(18,2),
		[RealizationAverageDay] decimal(18,2),
		[EfficiencyPercent] decimal(18,2),
		[EfficiencyDeviation] decimal(18,2),
		[RevenuePlanned] decimal(18,2),
		[RevenueReal] decimal(18,2),
		[RevenueDeviation] decimal(18,2),
		[RevenueDeviationPercent] decimal(18,2),
		[AverageRate] decimal(18,2)
	)

	DECLARE @EmployeeDeviations TABLE
	(
		[EmployeeID] Tidentifier,			
		[RealizationPositiveDeviation] decimal(18,2),
		[RealizationNegativeDeviation] decimal(18,2),			
		[RevenuePositiveDeviation] decimal(18,2),
		[RevenueNegativeDeviation] decimal(18,2)
	)

	--*******************************************************
	--Формирование списка сотрудников
	DECLARE @EmployeesList TABLE 
	(
		[EmployeeID] int,		
		[DivisionID] int,
		[QualificationID] int
	)
	INSERT INTO @EmployeesList
	SELECT 
		* 
	FROM 
		[dbo].[fnReportTimeSellAnalysisByEmployeesGetEmployeesList]
		(
			@OwnCompanyID,
			@DivisionSelector,			
			@DivisionID,
			@QualificationSelector,			
			@QualificationID,
			@EmployeeState,
			@EmployeeSelector,
			@EmployeeXml,
			@PeriodStart,
			@PeriodEnd,
			@IncludingSentToApprove
		)



	--*******************************************************
	--Обработка данных по сотрудникам
	DECLARE
		@tmpEmployeeID int,
		@tmpDivisionID int,
		@tmpQualificationID int

	DECLARE employeesCursor CURSOR FAST_FORWARD LOCAL FOR
		SELECT
			[EmployeeID],
			[DivisionID],
			[QualificationID]
		FROM
			@EmployeesList
	OPEN [employeesCursor]
	FETCH NEXT FROM [employeesCursor] INTO @tmpEmployeeID,@tmpDivisionID,@tmpQualificationID

	WHILE @@fetch_status = 0
	BEGIN
		
		 DECLARE
			@UtilizationPlanned decimal(18,2),
			@UtilizationReal decimal(18,2),
			@UtilizationDeviation decimal(18,2),
			@UtilizationDeviationPercent decimal(18,2),
			@UtilizationAverageMonth decimal(18,2),
			@UtilizationAverageDay decimal(18,2),
			@RealizationPlanned decimal(18,2),
			@RealizationReal decimal(18,2),
			@RealizationDeviation decimal(18,2),
			@RealizationDeviationPercent decimal(18,2),
			@RealizationAverageMonth decimal(18,2),
			@RealizationAverageDay decimal(18,2),
			@EfficiencyPercent decimal(18,2),
			@EfficiencyDeviation decimal(18,2),
			@RevenuePlanned decimal(18,2),
			@RevenueReal decimal(18,2),
			@RevenueDeviation decimal(18,2),
			@RevenueDeviationPercent decimal(18,2),
			@AverageRate decimal(18,2)	


		--*******************************************************
		--Заполнение нормативов
		EXEC ReportTimeSellAnalysisByEmployeesGetPlannedValuesList 
			@tmpEmployeeID, 
			@PeriodStart,
			@PeriodEnd, 
			@UtilizationPlanned OUTPUT, 
			@RealizationPlanned OUTPUT, 
			@RevenuePlanned OUTPUT
			

		--*******************************************************
		--Заполнение фактических данных	

		DECLARE
			@RealizationPositiveDeviation decimal(18,2),
			@RealizationNegativeDeviation decimal(18,2),			
			@RevenuePositiveDeviation decimal(18,2),
			@RevenueNegativeDeviation decimal(18,2)
	
		EXEC ReportTimeSellAnalysisByEmployeesGetRealValuesList
			@tmpEmployeeID,
			@PeriodStart, 
			@PeriodEnd,
			@IncludingSentToApprove,
			@StateOfFinanceDocuments,
			@CurrencySelector,
			@StandardEfficiency,
			@UtilizationReal OUTPUT,
			@UtilizationAverageMonth OUTPUT,
			@UtilizationAverageDay OUTPUT,
			@RealizationReal OUTPUT,
			@RealizationPositiveDeviation OUTPUT,
			@RealizationNegativeDeviation OUTPUT,
			@RealizationAverageMonth OUTPUT,
			@RealizationAverageDay OUTPUT,
			@RevenueReal OUTPUT,
			@RevenuePositiveDeviation OUTPUT,
			@RevenueNegativeDeviation OUTPUT,
			@EfficiencyPercent OUTPUT,
			@EfficiencyDeviation OUTPUT,
			@AverageRate OUTPUT
		
		INSERT INTO @EmployeeDeviations
		SELECT
			@tmpEmployeeID,
			@RealizationPositiveDeviation,
			@RealizationNegativeDeviation,
			@RevenuePositiveDeviation,
			@RevenueNegativeDeviation		

	
		--*******************************************************
		--Заполнение вычисляемых полей	
		EXEC ReportTimeSellAnalysisByEmployeesCalculateComputedValues
			@UtilizationPlanned,
			@UtilizationReal,
			@RealizationPlanned,
			@RealizationReal,
			@RealizationPositiveDeviation,
			@RevenuePlanned,
			@RevenueReal,			
			@UtilizationDeviation OUTPUT,
			@UtilizationDeviationPercent OUTPUT,
			@RealizationDeviation OUTPUT,
			@RealizationDeviationPercent OUTPUT,			
			@RevenueDeviation OUTPUT,
			@RevenueDeviationPercent OUTPUT


		
		--*******************************************************
		--Вставка строки по сотруднику во временную таблицу
		INSERT INTO @MainTable
		SELECT 
			0,
			@tmpEmployeeID,		
			@tmpDivisionID,
			@tmpQualificationID,
			@UtilizationPlanned,
			@UtilizationReal,
			@UtilizationDeviation,
			@UtilizationDeviationPercent,
			@UtilizationAverageMonth,
			@UtilizationAverageDay,
			@RealizationPlanned,
			@RealizationReal,
			@RealizationDeviation,
			@RealizationDeviationPercent,
			@RealizationAverageMonth,
			@RealizationAverageDay,
			@EfficiencyPercent,
			@EfficiencyDeviation,
			@RevenuePlanned,
			@RevenueReal,
			@RevenueDeviation,
			@RevenueDeviationPercent,
			@AverageRate	
			



		FETCH NEXT FROM [employeesCursor] INTO @tmpEmployeeID,@tmpDivisionID,@tmpQualificationID

	END
	CLOSE [employeesCursor]
	DEALLOCATE [employeesCursor]
	
	--*******************************************************
	--Итоги и группировка
	
	DECLARE
		@RowsCount int
	SELECT
		@RowsCount = COUNT([EmployeeID])
	FROM
		@MainTable

	--SELECT * FROM @MainTable		

	--Добавление строки общих итогов
	INSERT INTO @MainTable
	SELECT
		2 AS [Order],
		NULL as [EmployeeID],			
		NULL AS [DivisionID],
		NULL AS [QualificationID],

		(SELECT SUM([MT1].[UtilizationPlanned]) FROM @MainTable AS [MT1]) AS [UtilizationPlanned],
		(SELECT SUM([MT1].[UtilizationReal]) FROM @MainTable AS [MT1]) AS [UtilizationReal],
		(SELECT SUM([MT1].[UtilizationReal]) FROM @MainTable AS [MT1]) 
		- 
		(SELECT SUM([MT1].[UtilizationPlanned]) FROM @MainTable AS [MT1]) AS [UtilizationDeviation],
		ROUND
		(
			(
				((SELECT SUM([MT1].[UtilizationReal]) FROM @MainTable AS [MT1]) - (SELECT SUM([MT1].[UtilizationPlanned]) FROM @MainTable AS [MT1]))
				/
				(SELECT SUM([MT1].[UtilizationPlanned]) FROM @MainTable AS [MT1])
			)*100
			,2
		) AS [UtilizationDeviationPercent],
		ROUND
		(
			(SELECT SUM(ROUND([MT1].[UtilizationAverageMonth],1)) FROM @MainTable AS [MT1] WHERE [UtilizationAverageMonth] IS NOT NULL)
			/
			(SELECT COUNT([MT1].[UtilizationAverageMonth]) FROM @MainTable AS [MT1] WHERE [UtilizationAverageMonth] IS NOT NULL)
			,2
		) AS [UtilizationAverageMonth],	
		ROUND
		(
			(SELECT SUM(ROUND([MT1].[UtilizationAverageDay],1)) FROM @MainTable AS [MT1] WHERE [UtilizationAverageDay] IS NOT NULL)
			/
			(SELECT COUNT([MT1].[UtilizationAverageDay]) FROM @MainTable AS [MT1] WHERE [UtilizationAverageDay] IS NOT NULL)
			,2
		) AS [UtilizationAverageDay],

		(SELECT SUM([MT1].[RealizationPlanned]) FROM @MainTable AS [MT1]) AS [RealizationPlanned],
		(SELECT SUM([MT1].[RealizationReal]) FROM @MainTable AS [MT1]) AS [RealizationReal],
		(SELECT SUM([MT1].[RealizationReal]) FROM @MainTable AS [MT1]) 
		- 
		(SELECT SUM([MT1].[RealizationPlanned]) FROM @MainTable AS [MT1]) AS [RealizationDeviation],
		ROUND
		(
			(
				((SELECT SUM([MT1].[RealizationReal]) FROM @MainTable AS [MT1]) - (SELECT SUM([MT1].[RealizationPlanned]) FROM @MainTable AS [MT1]))
				/
				(SELECT SUM([MT1].[RealizationPlanned]) FROM @MainTable AS [MT1])
			)*100
			,2
		) AS [RealizationDeviationPercent],		
		ROUND
		(
			(SELECT SUM(ROUND([MT1].[RealizationAverageMonth],1)) FROM @MainTable AS [MT1] WHERE [RealizationAverageMonth] IS NOT NULL)
			/
			(SELECT COUNT([MT1].[RealizationAverageMonth]) FROM @MainTable AS [MT1] WHERE [RealizationAverageMonth] IS NOT NULL)
			,2
		) AS [RealizationAverageMonth],	
		ROUND
		(
			(SELECT SUM(ROUND([MT1].[RealizationAverageDay],1)) FROM @MainTable AS [MT1] WHERE [RealizationAverageDay] IS NOT NULL)
			/
			(SELECT COUNT([MT1].[RealizationAverageDay]) FROM @MainTable AS [MT1] WHERE [RealizationAverageDay] IS NOT NULL)
			,2
		) AS [RealizationAverageDay],

		ROUND(((SUM([RealizationReal]) - SUM([ED].[RealizationPositiveDeviation]))/SUM([UtilizationReal]))*100,2) AS [EfficiencyPercent],
		ROUND(((SUM([RealizationReal]) - SUM([ED].[RealizationPositiveDeviation]))/SUM([UtilizationReal]))*100,2) - @StandardEfficiency AS [EfficiencyDeviation],

		(SELECT SUM([MT1].[RevenuePlanned]) FROM @MainTable AS [MT1]) AS [RevenuePlanned],
		(SELECT SUM([MT1].[RevenueReal]) FROM @MainTable AS [MT1]) AS [RevenueReal],
		(SELECT SUM([MT1].[RevenueReal]) FROM @MainTable AS [MT1]) 
		- 
		(SELECT SUM([MT1].[RevenuePlanned]) FROM @MainTable AS [MT1]) AS [RevenueDeviation],
		ROUND
		(
			(
				((SELECT SUM([MT1].[RevenueReal]) FROM @MainTable AS [MT1]) - (SELECT SUM([MT1].[RevenuePlanned]) FROM @MainTable AS [MT1]))
				/
				(SELECT SUM([MT1].[RevenuePlanned]) FROM @MainTable AS [MT1])
			)*100
			,2
		) AS [RevenueDeviationPercent],

		ROUND
		(
			(SELECT SUM([MT1].[AverageRate]) FROM @MainTable AS [MT1] WHERE [AverageRate] IS NOT NULL)
			/
			(SELECT COUNT([MT1].[AverageRate]) FROM @MainTable AS [MT1] WHERE [AverageRate] IS NOT NULL)
			,2
		)
	FROM
		@MainTable,
		@EmployeeDeviations AS [ED]

	--Отклонение вверх
	INSERT INTO @MainTable
	SELECT
		3 AS [Order],
		NULL as [EmployeeID],		
		NULL AS [DivisionID],
		NULL AS [QualificationID],

		NULL AS [UtilizationPlanned],
		NULL AS [UtilizationReal],
		NULL AS [UtilizationDeviation],
		NULL AS [UtilizationDeviationPercent],
		NULL AS [UtilizationAverageMonth],
		NULL AS [UtilizationAverageDay],

		NULL AS [RealizationPlanned],
		SUM([ED].[RealizationPositiveDeviation]) AS [RealizationReal],
		NULL AS [RealizationDeviation],
		NULL AS [RealizationDeviationPercent],
		NULL AS [RealizationAverageMonth],
		NULL AS [RealizationAverageDay],

		--ROUND(SUM([EfficiencyPercent])/@RowsCount,2) 
		NULL AS [EfficiencyPercent],
		NULL AS [EfficiencyDeviation],

		NULL AS [RevenuePlanned],
		SUM([ED].[RevenuePositiveDeviation]) AS [RevenueReal],
		NULL AS [RevenueDeviation],
		NULL AS [RevenueDeviationPercent],

		NULL AS [AverageRate]			
	FROM
		@EmployeeDeviations AS [ED]

	--Отклонение вниз
	INSERT INTO @MainTable
	SELECT
		4 AS [Order],
		NULL as [EmployeeID],		
		NULL AS [DivisionID],
		NULL AS [QualificationID],

		NULL AS [UtilizationPlanned],
		NULL AS [UtilizationReal],
		NULL AS [UtilizationDeviation],
		NULL AS [UtilizationDeviationPercent],
		NULL AS [UtilizationAverageMonth],
		NULL AS [UtilizationAverageDay],

		NULL AS [RealizationPlanned],
		SUM([ED].[RealizationNegativeDeviation]) AS [RealizationReal],
		NULL AS [RealizationDeviation],
		NULL AS [RealizationDeviationPercent],
		NULL AS [RealizationAverageMonth],
		NULL AS [RealizationAverageDay],

		--ROUND(SUM([EfficiencyPercent])/@RowsCount,2) 
		NULL AS [EfficiencyPercent],
		NULL AS [EfficiencyDeviation],

		NULL AS [RevenuePlanned],
		SUM([ED].[RevenueNegativeDeviation]) AS [RevenueReal],
		NULL AS [RevenueDeviation],
		NULL AS [RevenueDeviationPercent],

		NULL AS [AverageRate]			
	FROM
		@EmployeeDeviations AS [ED]

	
	IF @GroupBy = 1 --Группировка по подразделениям
	BEGIN
		;
		WITH [DivisionSubtotals]
		(
			[Order],
			[EmployeeID],
			[DivisionID],
			[QualificationID],
			[UtilizationPlanned],
			[UtilizationReal],
			[UtilizationDeviation],
			[UtilizationDeviationPercent],
			[UtilizationAverageMonth],
			[UtilizationAverageDay],
			[RealizationPlanned],
			[RealizationReal],
			[RealizationDeviation],
			[RealizationDeviationPercent],
			[RealizationAverageMonth],
			[RealizationAverageDay],
			[EfficiencyPercent],
			[EfficiencyDeviation],
			[RevenuePlanned],
			[RevenueReal],
			[RevenueDeviation],
			[RevenueDeviationPercent],
			[AverageRate] 
		)
		AS
		(
			SELECT
				1 AS [Order],
				NULL as [EmployeeID],			
				[DivisionID],
				NULL AS [QualificationID],

				SUM([UtilizationPlanned]) AS [UtilizationPlanned],
				SUM([UtilizationReal]) AS [UtilizationReal],
				SUM([UtilizationReal]) - SUM([UtilizationPlanned]) AS [UtilizationDeviation],
				ROUND(((SUM([UtilizationReal]) - SUM([UtilizationPlanned]))/SUM([UtilizationPlanned]))*100,2) AS [UtilizationDeviationPercent],
				ROUND
				(
					(SELECT SUM(ROUND([MT1].[UtilizationAverageMonth],1)) FROM @MainTable AS [MT1] WHERE [UtilizationAverageMonth] IS NOT NULL AND [MT1].[DivisionID] = [@MainTable].[DivisionID])
					/
					(SELECT COUNT([MT1].[UtilizationAverageMonth]) FROM @MainTable AS [MT1] WHERE [UtilizationAverageMonth] IS NOT NULL AND [MT1].[DivisionID] = [@MainTable].[DivisionID])
					,2
				) AS [UtilizationAverageMonth],	
				ROUND
				(
					(SELECT SUM(ROUND([MT1].[UtilizationAverageDay],1)) FROM @MainTable AS [MT1] WHERE [UtilizationAverageDay] IS NOT NULL AND [MT1].[DivisionID] = [@MainTable].[DivisionID])
					/
					(SELECT COUNT([MT1].[UtilizationAverageDay]) FROM @MainTable AS [MT1] WHERE [UtilizationAverageDay] IS NOT NULL AND [MT1].[DivisionID] = [@MainTable].[DivisionID])
					,2
				) AS [UtilizationAverageDay],

				SUM([RealizationPlanned]) AS [RealizationPlanned],
				SUM([RealizationReal]) AS [RealizationReal],
				SUM([RealizationReal]) - SUM([RealizationPlanned]) AS [RealizationDeviation],
				ROUND(((SUM([RealizationReal]) - SUM([RealizationPlanned]))/SUM([RealizationPlanned]))*100,2) AS [RealizationDeviationPercent],
				ROUND
				(
					(SELECT SUM(ROUND([MT1].[RealizationAverageMonth],1)) FROM @MainTable AS [MT1] WHERE [RealizationAverageMonth] IS NOT NULL AND [MT1].[DivisionID] = [@MainTable].[DivisionID])
					/
					(SELECT COUNT([MT1].[RealizationAverageMonth]) FROM @MainTable AS [MT1] WHERE [RealizationAverageMonth] IS NOT NULL AND [MT1].[DivisionID] = [@MainTable].[DivisionID])
					,2
				) AS [RealizationAverageMonth],	
				ROUND
				(
					(SELECT SUM(ROUND([MT1].[RealizationAverageDay],1)) FROM @MainTable AS [MT1] WHERE [RealizationAverageDay] IS NOT NULL AND [MT1].[DivisionID] = [@MainTable].[DivisionID])
					/
					(SELECT COUNT([MT1].[RealizationAverageDay]) FROM @MainTable AS [MT1] WHERE [RealizationAverageDay] IS NOT NULL AND [MT1].[DivisionID] = [@MainTable].[DivisionID])
					,2
				) AS [RealizationAverageDay],

				NULL AS [EfficiencyPercent],
				NULL AS [EfficiencyDeviation],

				SUM([RevenuePlanned]) AS [RevenuePlanned],
				SUM([RevenueReal]) AS [RevenueReal],
				SUM([RevenueReal]) - SUM([RevenuePlanned]) AS [RevenueDeviation],
				ROUND(((SUM([RevenueReal]) - SUM([RevenuePlanned]))/SUM([RevenuePlanned]))*100,2) AS [RevenueDeviationPercent],

				ROUND
				(
					SUM([AverageRate])
					/
					(
						SELECT 
							COUNT([MT1].[AverageRate]) 
						FROM 
							@MainTable AS [MT1] 
						WHERE 
							[MT1].[AverageRate] IS NOT NULL 
							AND 
							[MT1].[DivisionID] = [@MainTable].[DivisionID]
					)
					,2
				) AS [AverageRate]				
			FROM
				@MainTable
			WHERE 
				[DivisionID] IS NOT NULL
			GROUP BY [DivisionID]
		)
		INSERT INTO @MainTable
		SELECT * FROM [DivisionSubtotals]
	END
	ELSE IF @GroupBy = 2 --Группировка по квалификациям
	BEGIN
		;
		WITH [QualificationSubtotals]
		(
			[Order],
			[EmployeeID],
			[DivisionID],
			[QualificationID],
			[UtilizationPlanned],
			[UtilizationReal],
			[UtilizationDeviation],
			[UtilizationDeviationPercent],
			[UtilizationAverageMonth],
			[UtilizationAverageDay],
			[RealizationPlanned],
			[RealizationReal],
			[RealizationDeviation],
			[RealizationDeviationPercent],
			[RealizationAverageMonth],
			[RealizationAverageDay],
			[EfficiencyPercent],
			[EfficiencyDeviation],
			[RevenuePlanned],
			[RevenueReal],
			[RevenueDeviation],
			[RevenueDeviationPercent],
			[AverageRate] 
		)
		AS
		(
			SELECT
				1 AS [Order],
				NULL AS [EmployeeID],			
				NULL AS [DivisionID],
				[QualificationID],

				SUM([UtilizationPlanned]) AS [UtilizationPlanned],
				SUM([UtilizationReal]) AS [UtilizationReal],
				SUM([UtilizationReal]) - SUM([UtilizationPlanned]) AS [UtilizationDeviation],
				ROUND(((SUM([UtilizationReal]) - SUM([UtilizationPlanned]))/SUM([UtilizationPlanned]))*100,2) AS [UtilizationDeviationPercent],
				ROUND
				(
					(SELECT SUM(ROUND([MT1].[UtilizationAverageMonth],1)) FROM @MainTable AS [MT1] WHERE [UtilizationAverageMonth] IS NOT NULL AND [MT1].[QualificationID] = [@MainTable].[QualificationID])
					/
					(SELECT COUNT([MT1].[UtilizationAverageMonth]) FROM @MainTable AS [MT1] WHERE [UtilizationAverageMonth] IS NOT NULL AND [MT1].[QualificationID] = [@MainTable].[QualificationID])
					,2
				) AS [UtilizationAverageMonth],	
				ROUND
				(
					(SELECT SUM(ROUND([MT1].[UtilizationAverageDay],1)) FROM @MainTable AS [MT1] WHERE [UtilizationAverageDay] IS NOT NULL AND [MT1].[QualificationID] = [@MainTable].[QualificationID])
					/
					(SELECT COUNT([MT1].[UtilizationAverageDay]) FROM @MainTable AS [MT1] WHERE [UtilizationAverageDay] IS NOT NULL AND [MT1].[QualificationID] = [@MainTable].[QualificationID])
					,2
				) AS [UtilizationAverageDay],

				SUM([RealizationPlanned]) AS [RealizationPlanned],
				SUM([RealizationReal]) AS [RealizationReal],
				SUM([RealizationReal]) - SUM([RealizationPlanned]) AS [RealizationDeviation],
				ROUND(((SUM([RealizationReal]) - SUM([RealizationPlanned]))/SUM([RealizationPlanned]))*100,2) AS [RealizationDeviationPercent],
				ROUND
				(
					(SELECT SUM(ROUND([MT1].[RealizationAverageMonth],1)) FROM @MainTable AS [MT1] WHERE [RealizationAverageMonth] IS NOT NULL AND [MT1].[QualificationID] = [@MainTable].[QualificationID])
					/
					(SELECT COUNT([MT1].[RealizationAverageMonth]) FROM @MainTable AS [MT1] WHERE [RealizationAverageMonth] IS NOT NULL AND [MT1].[QualificationID] = [@MainTable].[QualificationID])
					,2
				) AS [RealizationAverageMonth],	
				ROUND
				(
					(SELECT SUM(ROUND([MT1].[RealizationAverageDay],1)) FROM @MainTable AS [MT1] WHERE [RealizationAverageDay] IS NOT NULL AND [MT1].[QualificationID] = [@MainTable].[QualificationID])
					/
					(SELECT COUNT([MT1].[RealizationAverageDay]) FROM @MainTable AS [MT1] WHERE [RealizationAverageDay] IS NOT NULL AND [MT1].[QualificationID] = [@MainTable].[QualificationID])
					,2
				) AS [RealizationAverageDay],

				NULL AS [EfficiencyPercent],
				NULL AS [EfficiencyDeviation],

				SUM([RevenuePlanned]) AS [RevenuePlanned],
				SUM([RevenueReal]) AS [RevenueReal],
				SUM([RevenueReal]) - SUM([RevenuePlanned]) AS [RevenueDeviation],
				ROUND(((SUM([RevenueReal]) - SUM([RevenuePlanned]))/SUM([RevenuePlanned]))*100,2) AS [RevenueDeviationPercent],

				ROUND
				(
					SUM([AverageRate])
					/
					(
						SELECT 
							COUNT([MT1].[AverageRate]) 
						FROM 
							@MainTable AS [MT1] 
						WHERE 
							[MT1].[AverageRate] IS NOT NULL 
							AND 
							[MT1].[QualificationID] = [@MainTable].[QualificationID]
					)
					,2
				) AS [AverageRate]			
			FROM
				@MainTable
			WHERE 
				[QualificationID] IS NOT NULL
			GROUP BY [QualificationID]
		)
		INSERT INTO @MainTable
		SELECT * FROM [QualificationSubtotals]
	END

	--SELECT * FROM @MainTable
	
	
	--*******************************************************
	--Округление
	UPDATE @MainTable
	SET [UtilizationDeviationPercent] = ROUND([UtilizationDeviationPercent],0),
		[RealizationDeviationPercent] = ROUND([RealizationDeviationPercent],0),
		[EfficiencyPercent] = ROUND([EfficiencyPercent],0),
		[EfficiencyDeviation] = ROUND([EfficiencyDeviation],0),
		[RevenueDeviationPercent] = ROUND([RevenueDeviationPercent],0)
		

	--*******************************************************
	--Smart Value
	DECLARE 
		@Scale int,
		@MaxValue decimal(18,2)

	SELECT
		@MaxValue = 
		CASE 
			WHEN ISNULL(MAX([RevenuePlanned]),0) > ISNULL(MAX([RevenueReal]),0) 
			THEN ISNULL(MAX([RevenuePlanned]),0)
			ELSE ISNULL(MAX([RevenueReal]),0)
		END
	FROM
		@MainTable	
		

	SELECT
		@Scale = [dbo].[fnReportCalculateSmartValue](@ScaleSelector,@MaxValue)
	

	UPDATE @MainTable
	SET [RevenuePlanned] = [RevenuePlanned]/@Scale,
		[RevenueReal] = [RevenueReal]/@Scale,
		[RevenueDeviation] = [RevenueDeviation]/@Scale

	

	--*******************************************************
	--Выборка данных

	--Шапка отчета
	EXEC ReportTimeSellAnalysisByEmployeesHeader 
			@OwnCompanyID, 
			@DivisionSelector, 
			@DivisionID, 
			@QualificationSelector, 
			@QualificationID, 
			@CurrencySelector, 
			@PeriodSelector,
			@PeriodName,
			@PeriodStart,
			@PeriodEnd,
			@ScaleSelector, 
			@Scale,
			@GroupBy,
			@StandardEfficiency


	--Основная таблица
	SELECT 
		[Order],
		CASE [Order]
			WHEN 0 THEN
				--Блок взят из процедуры ReportProjectTimeAllocationByEmployee 
				CASE
					WHEN (([E].[LastName] IS NULL) OR ([E].[LastName] = ''))
						THEN [E].[FileAs]  -- Если нет Фамилии, показать поле "Хранить как"
					WHEN ((([E].[FirstName] IS NULL) OR ([E].[FirstName] = '')) 
						AND (([E].[MiddleName] IS NULL) OR ([E].[MiddleName] = '')))
						THEN [E].[LastName]	-- Если отсутствуют имя и очество, то выводить фамилию
					WHEN (([E].[FirstName] IS NULL) OR ([E].[FirstName] = ''))
						THEN [E].[LastName]	-- Если отсутствует имя, то выводить фамилию
					WHEN (([E].[MiddleName] IS NULL) OR ([E].[MiddleName] = ''))	-- Если отсутствует отчество, то выводить фамилию и инициал имени
						THEN [E].[LastName] + ' ' + SUBSTRING(CAST([E].[FirstName] as varchar), 1, 1) + '.'
					ELSE
						[E].[LastName] + ' ' + SUBSTRING([E].[FirstName], 1, 1) + '.' + SUBSTRING([E].[MiddleName], 1, 1) + '.'
				END
			WHEN 1 THEN NULL
			WHEN 2 THEN 'ИТОГО:'
			WHEN 3 THEN 'Отклонение вверх:'
			WHEN 4 THEN 'Отклонение вниз:'
		END AS [EmployeeFileAs],		
		CASE
			WHEN [MainTable].[DivisionID] = -1 THEN '(не указано)'
			ELSE
				CASE
					WHEN [D].[FileAs] = [D].[Code] THEN [D].[Code]
					ELSE [D].[Code] + ' / ' + [D].[FileAs]
				END
		END AS [DivisionFileAs],
		CASE
			WHEN [MainTable].[QualificationID] = -1 THEN '(не указана)'
			ELSE
				CASE
					WHEN [Q].[FileAs] = [Q].[Code] THEN [Q].[Code]
					ELSE [Q].[Code] + ' / ' + [Q].[FileAs]
				END
		END AS [QualificationFileAs],		
		[UtilizationPlanned],
		[UtilizationReal],
		[UtilizationDeviation],
		[UtilizationDeviationPercent],
		[UtilizationAverageMonth],
		[UtilizationAverageDay],
		[RealizationPlanned],
		[RealizationReal],
		[RealizationDeviation],
		[RealizationDeviationPercent],
		[RealizationAverageMonth],
		[RealizationAverageDay],
		[EfficiencyPercent],
		[EfficiencyDeviation],
		[RevenuePlanned],
		[RevenueReal],
		[RevenueDeviation],
		[RevenueDeviationPercent],
		[AverageRate] 
	FROM 
		@MainTable AS [MainTable]
		LEFT JOIN [Employee] AS [E]
			ON [E].[ID] = [MainTable].[EmployeeID]
		LEFT JOIN [Division] AS [D]
			ON [D].[ID] = [MainTable].[DivisionID]
		LEFT JOIN [Qualification] AS [Q]
			ON [Q].[ID] = [MainTable].[QualificationID]
	ORDER BY
		CASE @GroupBy
			WHEN 1 THEN 
				CASE 
					WHEN [D].[FileAs] IS NULL THEN 'яяяяяяя'
					ELSE 
						CASE
							WHEN [D].[FileAs] = [D].[Code] THEN [D].[Code]
							ELSE [D].[FileAs] + ' / ' + [D].[Code]
						END
				END
			WHEN 2 THEN 
				CASE 
					WHEN [Q].[FileAs] IS NULL THEN 'яяяяяяя'
					ELSE 
						CASE
							WHEN [Q].[FileAs] = [Q].[Code] THEN [Q].[Code]
							ELSE [Q].[FileAs] + ' / ' + [Q].[Code]
						END
				END
		END, 
		[MainTable].[Order],
		CASE [Order]
			WHEN 0 THEN [E].[FileAs]
			WHEN 1 THEN NULL
			WHEN 2 THEN 'ИТОГО:'
			WHEN 3 THEN 'Отклонение вверх:'
			WHEN 4 THEN 'Отклонение вниз'
		END	



	--График "Время утилизации и реализации"
	SELECT
		--Блок взят из процедуры ReportProjectTimeAllocationByEmployee 
		CASE
			WHEN (([E].[LastName] IS NULL) OR ([E].[LastName] = ''))
				THEN [E].[FileAs]  -- Если нет Фамилии, показать поле "Хранить как"
			WHEN ((([E].[FirstName] IS NULL) OR ([E].[FirstName] = '')) 
				AND (([E].[MiddleName] IS NULL) OR ([E].[MiddleName] = '')))
				THEN [E].[LastName]	-- Если отсутствуют имя и очество, то выводить фамилию
			WHEN (([E].[FirstName] IS NULL) OR ([E].[FirstName] = ''))
				THEN [E].[LastName]	-- Если отсутствует имя, то выводить фамилию
			WHEN (([E].[MiddleName] IS NULL) OR ([E].[MiddleName] = ''))	-- Если отсутствует отчество, то выводить фамилию и инициал имени
				THEN [E].[LastName] + ' ' + SUBSTRING(CAST([E].[FirstName] as varchar), 1, 1) + '.'
			ELSE
				[E].[LastName] + ' ' + SUBSTRING([E].[FirstName], 1, 1) + '.' + SUBSTRING([E].[MiddleName], 1, 1) + '.'
		END AS [FileAs],
		[MT].[UtilizationReal],
		[MT].[RealizationReal]
	FROM
		@MainTable AS [MT]
		INNER JOIN [Employee] AS [E]
			ON [E].[ID] = [MT].[EmployeeID]
	WHERE 
		[Order] = 0
	ORDER BY 
		[UtilizationReal] DESC

	--График "Отклонение утилизации и реализации от плана, %"
	SELECT
		--Блок взят из процедуры ReportProjectTimeAllocationByEmployee 
		CASE
			WHEN (([E].[LastName] IS NULL) OR ([E].[LastName] = ''))
				THEN [E].[FileAs]  -- Если нет Фамилии, показать поле "Хранить как"
			WHEN ((([E].[FirstName] IS NULL) OR ([E].[FirstName] = '')) 
				AND (([E].[MiddleName] IS NULL) OR ([E].[MiddleName] = '')))
				THEN [E].[LastName]	-- Если отсутствуют имя и очество, то выводить фамилию
			WHEN (([E].[FirstName] IS NULL) OR ([E].[FirstName] = ''))
				THEN [E].[LastName]	-- Если отсутствует имя, то выводить фамилию
			WHEN (([E].[MiddleName] IS NULL) OR ([E].[MiddleName] = ''))	-- Если отсутствует отчество, то выводить фамилию и инициал имени
				THEN [E].[LastName] + ' ' + SUBSTRING(CAST([E].[FirstName] as varchar), 1, 1) + '.'
			ELSE
				[E].[LastName] + ' ' + SUBSTRING([E].[FirstName], 1, 1) + '.' + SUBSTRING([E].[MiddleName], 1, 1) + '.'
		END AS [FileAs],
		[MT].[UtilizationDeviationPercent],
		[MT].[RealizationDeviationPercent]
	FROM 
		@MainTable AS [MT]
		INNER JOIN [Employee] AS [E]
			ON [E].[ID] = [MT].[EmployeeID]
	WHERE 
		[Order] = 0
	ORDER BY 
		[UtilizationReal] DESC

	--График "Эффективность сотрудников"
	SELECT
		--Блок взят из процедуры ReportProjectTimeAllocationByEmployee 
		CASE
			WHEN (([E].[LastName] IS NULL) OR ([E].[LastName] = ''))
				THEN [E].[FileAs]  -- Если нет Фамилии, показать поле "Хранить как"
			WHEN ((([E].[FirstName] IS NULL) OR ([E].[FirstName] = '')) 
				AND (([E].[MiddleName] IS NULL) OR ([E].[MiddleName] = '')))
				THEN [E].[LastName]	-- Если отсутствуют имя и очество, то выводить фамилию
			WHEN (([E].[FirstName] IS NULL) OR ([E].[FirstName] = ''))
				THEN [E].[LastName]	-- Если отсутствует имя, то выводить фамилию
			WHEN (([E].[MiddleName] IS NULL) OR ([E].[MiddleName] = ''))	-- Если отсутствует отчество, то выводить фамилию и инициал имени
				THEN [E].[LastName] + ' ' + SUBSTRING(CAST([E].[FirstName] as varchar), 1, 1) + '.'
			ELSE
				[E].[LastName] + ' ' + SUBSTRING([E].[FirstName], 1, 1) + '.' + SUBSTRING([E].[MiddleName], 1, 1) + '.'
		END AS [FileAs],
		[MT].[EfficiencyPercent]
	FROM 
		@MainTable AS [MT]
		INNER JOIN [Employee] AS [E]
			ON [E].[ID] = [MT].[EmployeeID]
	WHERE 
		[Order] = 0
	ORDER BY 
		[EfficiencyPercent]

	--График "Стоимость"
	SELECT
		--Блок взят из процедуры ReportProjectTimeAllocationByEmployee 
		CASE
			WHEN (([E].[LastName] IS NULL) OR ([E].[LastName] = ''))
				THEN [E].[FileAs]  -- Если нет Фамилии, показать поле "Хранить как"
			WHEN ((([E].[FirstName] IS NULL) OR ([E].[FirstName] = '')) 
				AND (([E].[MiddleName] IS NULL) OR ([E].[MiddleName] = '')))
				THEN [E].[LastName]	-- Если отсутствуют имя и очество, то выводить фамилию
			WHEN (([E].[FirstName] IS NULL) OR ([E].[FirstName] = ''))
				THEN [E].[LastName]	-- Если отсутствует имя, то выводить фамилию
			WHEN (([E].[MiddleName] IS NULL) OR ([E].[MiddleName] = ''))	-- Если отсутствует отчество, то выводить фамилию и инициал имени
				THEN [E].[LastName] + ' ' + SUBSTRING(CAST([E].[FirstName] as varchar), 1, 1) + '.'
			ELSE
				[E].[LastName] + ' ' + SUBSTRING([E].[FirstName], 1, 1) + '.' + SUBSTRING([E].[MiddleName], 1, 1) + '.'
		END AS [FileAs],
		[MT].[RevenuePlanned],
		[MT].[RevenueReal]
	FROM 
		@MainTable AS [MT]
		INNER JOIN [Employee] AS [E]
			ON [E].[ID] = [MT].[EmployeeID]
	WHERE 
		[Order] = 0
	ORDER BY 
		[RevenueReal]

	--График "Средняя ставка"
	SELECT 
		@Value = [AverageRate]
	FROM 
		@MainTable
	WHERE 
		[Order] = 2
	
	SELECT
		--Блок взят из процедуры ReportProjectTimeAllocationByEmployee 
		CASE
			WHEN (([E].[LastName] IS NULL) OR ([E].[LastName] = ''))
				THEN [E].[FileAs]  -- Если нет Фамилии, показать поле "Хранить как"
			WHEN ((([E].[FirstName] IS NULL) OR ([E].[FirstName] = '')) 
				AND (([E].[MiddleName] IS NULL) OR ([E].[MiddleName] = '')))
				THEN [E].[LastName]	-- Если отсутствуют имя и очество, то выводить фамилию
			WHEN (([E].[FirstName] IS NULL) OR ([E].[FirstName] = ''))
				THEN [E].[LastName]	-- Если отсутствует имя, то выводить фамилию
			WHEN (([E].[MiddleName] IS NULL) OR ([E].[MiddleName] = ''))	-- Если отсутствует отчество, то выводить фамилию и инициал имени
				THEN [E].[LastName] + ' ' + SUBSTRING(CAST([E].[FirstName] as varchar), 1, 1) + '.'
			ELSE
				[E].[LastName] + ' ' + SUBSTRING([E].[FirstName], 1, 1) + '.' + SUBSTRING([E].[MiddleName], 1, 1) + '.'
		END AS [FileAs],
		[MT].[AverageRate],
		@Value AS [CompanyAverage]
	FROM 
		@MainTable AS [MT]
		INNER JOIN [Employee] AS [E]
			ON [E].[ID] = [MT].[EmployeeID]
	WHERE 
		[Order] = 0
	ORDER BY 
		[AverageRate]

	RETURN 0
GO

GRANT EXEC ON ReportTimeSellAnalysisByEmployees TO PUBLIC

GO

